#include "testlib.h"
using namespace std;

const std::string _grader_OK = "OK";
const std::string _grader_SV = "SV";
const std::string _grader_WA = "WA";
const std::string _grader_FAIL = "FAIL";

void readSecret(InStream& in, std::string secret)
{
  	string in_sec = in.readWord();
    if (in_sec != secret)
        in.quitf(_wa, "secret mismatch %s", in_sec.c_str());
    in.readEoln();
}

void readBothSecrets(std::string secret)
{
    readSecret(ans, secret);
    readSecret(ouf, secret);
}

/// 1 -> 1st, 2 -> 2nd, 3 -> 3rd, 4 -> 4th, ...
std::string englishTh(int x)
{
    char c[100];
    sprintf(c, "%d%s", x, englishEnding(x).c_str());
    return c;
}

void readGraderResult(InStream& in) {

    std::string result = in.readWord();
    in.readEoln();
    if (result == _grader_OK)
        return;
    if (result == _grader_SV)
    {
        if (in.seekEof())
           in.quitf(_wa, "Security violation in grader");
        std::string msg = in.readLine();
        in.quitf(_wa, "Security violation in grader: %s", msg.c_str());
    }
    if (result == _grader_WA)
    {
        if (in.seekEof())
            in.quitf(_wa, "WA in grader");
        std::string msg = in.readLine();
        in.quitf(_wa, "WA in grader: %s", msg.c_str());
    }
    if (result == _grader_FAIL)
    {
        if (in.seekEof())
            in.quitf(_fail, "FAIL in grader");
        std::string msg = in.readLine();
        in.quitf(_fail, "FAIL in grader: %s", msg.c_str());
    }
    in.quitf(_fail, "Unknown grader result");
}

void readBothGraderResults()
{
    readGraderResult(ans);
    readGraderResult(ouf);
}


/// Compares the tokens of two lines
void compareTokens(int lineNo, std::string a, std::string b, char separator=' ')
{
    std::vector<std::string> toka = tokenize(a, separator);
    std::vector<std::string> tokb = tokenize(b, separator);
    if (toka == tokb)
        return;
    if (toka.size() != tokb.size())
        quit(_wa, "");
    for (int i=0; i<int(toka.size()); i++)
        if (toka[i] != tokb[i])
        	quit(_wa, "");
    std::string dif = format("%s lines differ - ", englishTh(lineNo).c_str());
    quitf(_fail, "%sbut I don't know why!", dif.c_str());
}

/// Compares the tokens of the remaining lines
NORETURN void compareRemainingLines(int lineNo=1)
{
    for (; !ans.eof(); lineNo++) 
    {
        std::string j = ans.readString();

        if (j == "" && ans.eof())
          break;
        
        std::string p = ouf.readString();

        compareTokens(lineNo, j, p);
    }
    quit(_ok, "");
}


void registerChecker(std::string probName, int argc, char* argv[])
{
    setName("checker for problem %s", probName.c_str());
    registerTestlibCmd(argc, argv);
}

using namespace std;

static string output_secret = "wji3t9iql4gwq1n-OUTPUT-s6f20p5z2lkxbj3";

int main(int argc, char * argv[])
{
	registerChecker("railroad", argc, argv);

	readBothSecrets(output_secret);
	readBothGraderResults();

	compareRemainingLines(3);
}
