/*
 * 다음과 같은 과정으로 채점된다고 가정해도 좋습니다.
 *
 * Pass 1:  ./railroad < %03d.in.txt > output1.txt
 * Pass 2:  ./railroad < output1.txt > output2.txt
 * Checker: ./checker %03d.in.txt output2.txt answer.txt
 */

#line 1 "/dev/null"
#include "railroad.h"
#include <vector>
#include <string>
#include <cassert>
#include <random>
#include <cstdio>
#include <map>
#include <algorithm>
#include <utility>

namespace {
    static const std::string input_secret = "jlsvaji35cwcxva-INPUT-gwl9ravworar2t1";
    static const std::string pipe_secret = "xesbjgyk879wak7-PIPE-o4pbvexzrdpcje2";
    static const std::string output_secret = "wji3t9iql4gwq1n-OUTPUT-s6f20p5z2lkxbj3";
    
    static std::mt19937 rng(52525);
    static FILE *pipe1;

    static void error1(std::string msg, std::string reason = "") {
		fprintf(pipe1, "%s\n", pipe_secret.c_str());
		fprintf(pipe1, "%s\n", msg.c_str());
		fprintf(pipe1, "%s\n", reason.c_str());
		fclose(pipe1);
		exit(0);
    }

    static void pass1() {
    	pipe1 = stdout;

    	int tests;
    	assert(1 == scanf("%d", &tests));
    
    	std::vector<std::vector<std::pair<int, int> > > T(tests), G(tests);
    	std::vector<int> N(tests), K(tests), X(tests, 0);
    
    	for(int t = 0; t < tests; t++) {
    		assert(2 == scanf("%d %d", &N[t], &K[t]));
    		for(int i = 0; i < N[t] - 1; i++) {
    			int a, b;
    			assert(2 == scanf("%d %d", &a, &b));
    			T[t].emplace_back(a, b);
    		}
    	}

    	fclose(stdin);
    
    	for(int t = 0; t < tests; t++) {
    		std::vector<std::pair<int, int> > edges = encode_map(N[t], K[t], X[t], T[t]);
    
    		if(X[t] < 1 || X[t] > N[t]) {
    			error1("WA", "invalid index");
    		}
    		if((int)edges.size() != K[t]) {
    			error1("WA", "invalid edge count - encode");
    		}
    
    		for(auto &i : T[t]) edges.push_back(i);
    
    		std::map<std::pair<int, int>, int> mp;
    		for(auto &i : edges) {
    			if(i.first > i.second) std::swap(i.first, i.second);
    			if(i.first == i.second || mp[i]) {
    				error1("WA", "invalid graph - encode");
    			}
    			mp[i] = 1;
    			if(i.first < 1 || i.first > N[t] || i.second < 1 || i.second > N[t]) {
    				error1("WA", "invalid edge - encode");
    			}
    
    			if(rng() % 2 == 1) std::swap(i.first, i.second);
    		}
    		std::shuffle(edges.begin(), edges.end(), rng);
    		G[t] = edges;
    	}

    	fprintf(pipe1, "%s\n", pipe_secret.c_str());
    	fprintf(pipe1, "OK\n");
    	fprintf(pipe1, "%d\n", tests);
    	for(int t = 0; t < tests; t++) {
    		fprintf(pipe1, "%d %d %d\n", N[t], K[t], X[t]);
    		for(auto &i : G[t]) {
    			fprintf(pipe1, "%d %d\n", i.first, i.second);
    		}
    	}

    	fclose(pipe1);
    }
    
    static void error2(std::string msg, std::string reason = "") {
    	printf("%s\n", output_secret.c_str());
    	printf("%s\n", msg.c_str());
    	printf("%s\n", reason.c_str());
    	fclose(stdout);
    	exit(0);
    }

    static void pass2() {
    	FILE *pipe2 = stdin;
    	char status[1000];

    	assert(1 == fscanf(pipe2, "%s\n", status));
    	if(std::string(status) != "OK") {
    		std::string reason = "";
			while(true) {
				int c = fgetc(pipe2);
				if(c == EOF || c=='\n')
					break;
				reason += char(c);
			}
			error2(status, reason);
    	}

    	int tests;
    	assert(1 == fscanf(pipe2, "%d", &tests));

    	std::vector<std::vector<std::pair<int, int> > > G(tests), Ans(tests);
    	std::vector<int> N(tests), K(tests), X(tests);

    	for(int t = 0; t < tests; t++) {
    		assert(3 == fscanf(pipe2, "%d %d %d", &N[t], &K[t], &X[t]));
    		for(int i = 0; i < N[t] + K[t] - 1; i++) {
    			int a, b;
    			assert(2 == fscanf(pipe2, "%d %d", &a, &b));
    			G[t].emplace_back(a, b);
    		}
    	}
    	fclose(pipe2);

       	std::vector<int> perm(tests);
    	for(int i = 0; i < tests; i++) perm[i] = i;
    	std::shuffle(perm.begin(), perm.end(), rng);
    
    	for(int _t = 0; _t < tests; _t++) {
    		int t = perm[_t];
    
    		std::vector<int> P(N[t] + 1), inv(N[t] + 1);
    
    		for(int i = 1; i <= N[t]; i++) P[i] = i;
    		std::shuffle(P.begin() + 1, P.begin() + N[t] + 1, rng);
    		for(int i = 1; i <= N[t]; i++) inv[P[i]] = i;
    
    		for(auto &i : G[t]) {
    			i.first = P[i.first];
    			i.second = P[i.second];
    		}
    
    		Ans[t] = decode_map(N[t], K[t], P[X[t]], G[t]);
    
    		if((int)Ans[t].size() != N[t] - 1) {
    			error2("WA", "invalid edge count - decode");
    		}
    
    		for(auto &i : Ans[t]) {
    			if(i.first < 1 || i.first > N[t] || i.second < 1 || i.second > N[t]) {
    				error2("WA", "invalid edge - decode");
    			}
    
    			i.first = inv[i.first];
    			i.second = inv[i.second];
    		}
    	}
    
    	printf("%s\n", output_secret.c_str());
    	printf("OK\n");
    
    	for(int t = 0; t < tests; t++) {
    	    printf("START %d\n", t+1);
    		for(auto &i : Ans[t]) {
    			if(i.first > i.second) std::swap(i.first, i.second);
    		}
    		std::sort(Ans[t].begin(), Ans[t].end());
    		for(auto &i : Ans[t]) {
    			printf("%d %d\n", i.first, i.second);
    		}
    		printf("END %d\n", t+1);
    	}
    }
}

int main() {
	char secret[1000];
	if(1 != scanf("%995s", secret)) error2("SV");
	if(std::string(secret) == input_secret) pass1();
	else if(std::string(secret) == pipe_secret) pass2();
	else error2("SV");
	return 0;
}
